/*
 * fd.attribs.c
 *
 * This file is part of the Form Designer.
 *
 * It contains some routines to deal with attributes of the objects.
 *
 * Written by Mark Overmars
 *
 * Version 2.1a
 * Date  Jan 24, 1992
 */

#include <string.h>
#include <stdio.h>
#include <gl/device.h>
#include "fd_main.h"
#include "fd_theforms.h"

/****************** FORMS AND CALL-BACKS ************************/

void setcolor_cb(FL_OBJECT *obj, long arg)
/* Call-back routine to get a color from the user */
{
  int col1 = fl_show_colormap(obj->col1);
  fl_set_object_color(obj,col1,col1);
}

/****************** GLOBAL INITIALIZATION  ************************/

static int atrinit = 0;

static void init_form()
/* Initializes the attributes form. */
{
  if (atrinit) return;
  atrinit = 1;

  fl_clear_choice(boxobj);
  fl_addto_choice(boxobj,"No box");
  fl_addto_choice(boxobj,"Up box");
  fl_addto_choice(boxobj,"Down box");
  fl_addto_choice(boxobj,"Flat box");
  fl_addto_choice(boxobj,"Border box");
  fl_addto_choice(boxobj,"Shadow box");
  fl_addto_choice(boxobj,"Frame box");
  fl_addto_choice(boxobj,"Rounded box");
  fl_addto_choice(boxobj,"Rflat box");
  fl_addto_choice(boxobj,"Rshadow box");

  fl_clear_choice(styleobj);
  fl_addto_choice(styleobj,FL_FONT_NAME_0);
  fl_addto_choice(styleobj,FL_FONT_NAME_1);
  fl_addto_choice(styleobj,FL_FONT_NAME_2);
  fl_addto_choice(styleobj,FL_FONT_NAME_3);
  fl_addto_choice(styleobj,FL_FONT_NAME_4);
  fl_addto_choice(styleobj,FL_FONT_NAME_5);
  fl_addto_choice(styleobj,FL_FONT_NAME_6);
  fl_addto_choice(styleobj,FL_FONT_NAME_7);
  fl_addto_choice(styleobj,FL_FONT_NAME_8);
  fl_addto_choice(styleobj,FL_FONT_NAME_9);
  fl_addto_choice(styleobj,FL_FONT_NAME_10);
  fl_addto_choice(styleobj,FL_FONT_NAME_11);
  fl_addto_choice(styleobj,FL_FONT_NAME_12);
  fl_addto_choice(styleobj,FL_FONT_NAME_13);
  fl_addto_choice(styleobj,FL_FONT_NAME_14);
  fl_addto_choice(styleobj,FL_FONT_NAME_15);

  fl_clear_choice(alignmentobj);
  fl_addto_choice(alignmentobj,"Top");
  fl_addto_choice(alignmentobj,"Bottom");
  fl_addto_choice(alignmentobj,"Left");
  fl_addto_choice(alignmentobj,"Right");
  fl_addto_choice(alignmentobj,"Center");

  fl_clear_choice(sizeobj);
  fl_addto_choice(sizeobj,"6");
  fl_addto_choice(sizeobj,"8 (small)");
  fl_addto_choice(sizeobj,"10");
  fl_addto_choice(sizeobj,"11 (normal)");
  fl_addto_choice(sizeobj,"16");
  fl_addto_choice(sizeobj,"20 (large)");
  fl_addto_choice(sizeobj,"24");
  fl_addto_choice(sizeobj,"32");

  fl_set_choice_fontsize(typeobj,FL_SMALL_FONT);
}

/********************* THE ACTUAL ROUTINES ****************************/

int change_object(FL_OBJECT *obj, int all)
/* Displays the form to change the attributes. all indicates whether
   all label, name, etc. should also be changed */
{
  FL_OBJECT *retobj;
  char objname[40], cbname[40], argname[40];
  int i;
  long dev;
  short val;

  init_form();

  /* Show only required parts */
  if (all)
  {
    fl_show_object(labelobj);
    fl_show_object(nameobj);
    fl_show_object(cbnameobj);
    fl_show_object(argobj);
  }
  else
  {
    fl_hide_object(labelobj);
    fl_hide_object(nameobj);
    fl_hide_object(cbnameobj);
    fl_hide_object(argobj);
  }

  /* Fill in list of types */
  fl_clear_choice(typeobj);
  if (obj->objclass != FL_BOX)
  {
    for (i=0; i< find_class_maxtype(obj->objclass);i++)
       fl_addto_choice(typeobj,find_type_name(obj->objclass,i));
    fl_set_choice(typeobj,obj->type+1);
  }

  /* Fil in settings */
  fl_set_choice(boxobj,obj->boxtype+1);
  fl_set_choice(styleobj,obj->lstyle + 1);
  fl_set_choice(alignmentobj,obj->align + 1);

  if (obj->lsize == 6.0) fl_set_choice(sizeobj,1);
  else if (obj->lsize == 8.0) fl_set_choice(sizeobj,2);
  else if (obj->lsize == 10.0) fl_set_choice(sizeobj,3);
  else if (obj->lsize == 11.0) fl_set_choice(sizeobj,4);
  else if (obj->lsize == 16.0) fl_set_choice(sizeobj,5);
  else if (obj->lsize == 20.0) fl_set_choice(sizeobj,6);
  else if (obj->lsize == 24.0) fl_set_choice(sizeobj,7);
  else if (obj->lsize == 32.0) fl_set_choice(sizeobj,8);
  else fl_set_choice(sizeobj,4);

  get_object_name(obj,objname,cbname,argname);
  fl_set_input(nameobj,objname);
  fl_set_input(cbnameobj,cbname);
  fl_set_input(argobj,argname);

  fl_set_input(labelobj,get_label(obj));

  fl_set_object_color(col1obj,obj->col1,obj->col1);
  fl_set_object_color(col2obj,obj->col2,obj->col2);
  fl_set_object_color(lcolobj,obj->lcol,obj->lcol);

  /* Do interaction */

  fl_deactivate_all_forms();
  fl_show_form(attrform,FL_PLACE_CENTER,FALSE,NULL);
  do {
    retobj = fl_do_forms();
    if (retobj == FL_EVENT)
      if ( ( dev = fl_qread(&val)) == REDRAW) redraw_the_form(1);
  } while (! (retobj == readyobj || retobj == cancelobj));
  fl_hide_form(attrform);
  fl_activate_all_forms();
  if (retobj == cancelobj) return FALSE;

  /* Read back form */

  obj->boxtype = fl_get_choice(boxobj)-1;
  obj->lstyle = fl_get_choice(styleobj)-1;
  obj->align = fl_get_choice(alignmentobj)-1;

  obj->col1 = col1obj->col1;
  obj->col2 = col2obj->col1;
  obj->lcol = lcolobj->col1;

  if (obj->objclass == FL_BOX)
    change_type (obj,obj->boxtype);
  else
    change_type (obj,fl_get_choice(typeobj)-1);
   
  if (fl_get_choice(sizeobj) == 1) obj->lsize = 6.0;
  else if (fl_get_choice(sizeobj) == 2) obj->lsize = 8.0;
  else if (fl_get_choice(sizeobj) == 3) obj->lsize = 10.0;
  else if (fl_get_choice(sizeobj) == 4) obj->lsize = 11.0;
  else if (fl_get_choice(sizeobj) == 5) obj->lsize = 16.0;
  else if (fl_get_choice(sizeobj) == 6) obj->lsize = 20.0;
  else if (fl_get_choice(sizeobj) == 7) obj->lsize = 24.0;
  else if (fl_get_choice(sizeobj) == 8) obj->lsize = 32.0;

  set_label(obj,fl_get_input(labelobj)); 
  set_object_name(obj, fl_get_input(nameobj),
                    fl_get_input(cbnameobj), fl_get_input(argobj) );
  return TRUE;
}

void set_attribs(FL_OBJECT *obj, int boxtype, int col1, int col2,
		int lcol, int align, float lsize, int lstyle, char label[])
/* Sets the attributes of an object */
{
  obj->boxtype = boxtype;
  obj->col1 = col1;
  obj->col2 = col2;
  obj->lcol = lcol;
  obj->align = align;
  obj->lsize = lsize;
  obj->lstyle = lstyle;
  fl_set_object_label(obj,label);
}

#define NL	10

char tmpstr[10000];	/* Limit on label length introduced here */

void set_label(FL_OBJECT *obj, char str[])
/* Sets the label, turning \n in NL */
{
  int i=0, j=0;
  do
    if (str[i] == '\\' && str[i+1] == 'n') {tmpstr[j++] = NL; i++;}
    else tmpstr[j++] = str[i];
  while (str[i++] != NULL);
  fl_set_object_label(obj,tmpstr);
}

char *get_label(FL_OBJECT *obj)
/* Read the label, turning NL into \n */
{
  int i=0, j=0;
  do
    if (obj->label[i] == NL) {tmpstr[j++] = '\\'; tmpstr[j++] = 'n';}
    else tmpstr[j++] = obj->label[i];
  while (obj->label[i++] != NULL);
  return tmpstr;
}

FL_OBJECT *copy_object(FL_OBJECT *obj)
/* Makes an exact copy of the object */
{
  char name[40], cbname[40], argname[40];
  FL_OBJECT *obj2 = add_an_object(obj->objclass,obj->type,obj->x,obj->y,obj->w,obj->h);
  get_object_name(obj,name,cbname,argname);
  set_object_name(obj2,name,cbname,argname);
  set_attribs(obj2,obj->boxtype,obj->col1,obj->col2,
                obj->lcol,obj->align,obj->lsize,obj->lstyle,obj->label);
  fl_delete_object(obj2);
  return obj2;
}

void change_type(FL_OBJECT *obj,int type)
/* Changes the type of an object by reconstructing it. A quite nasty
   procedure that delves into the form structure in a bad way. */
{
  FL_OBJECT *ttt;
  if (obj->type == type) return;
  /* Create the new object. */
  ttt = add_an_object(obj->objclass,type,obj->x,obj->y,obj->w,obj->h);
  /* Remove it from the form */
  fl_delete_object(ttt);
  /* Set the attributes */
  set_attribs(ttt,obj->boxtype,obj->col1,obj->col2,
                obj->lcol,obj->align,obj->lsize,obj->lstyle,obj->label);
  /* Place it at the correct place in the form. */
  ttt->prev = obj->prev;
  ttt->next = obj->next;
  ttt->form = obj->form;
  /* Correct the object focus if required. */
  if (obj->focus) fl_set_object_focus(obj->form,ttt);
  /* Keep the pointer */
  *obj = *ttt;
}
